#####################################################
#Sanchez,J.Time Series for Data Scientists. CUP 2023 #
# Chapter 2 case study                               #
#  Based on paper and programs  at                   #
# https://doi.org/10.1080/10691898.2017.1286960      #
#   Last modified:3/6/2024                           #
######################################################

#install.packages("xts") # commented out
#install.packages("zoo") # commented out
#install.packages("dygraphs") # commented out
library(xts)
library(zoo)
library(dygraphs)

## read the monthly chemical time series 
chemicals<-read.csv("Lake_Mead_chemical_time_series.csv",
                      header=T,sep=",")
chemicals
# Notice that the date looks different from other datasets
# we have seen in Chapters 1 and 2. 

attach(chemicals)

date= seq.Date(from=as.Date("2003-03-15"),
                     to=as.Date("2007-12-15"),
                     by="month")
class(date)
date 
meprobamate=meprobamate
meprobamate

detach(chemicals)

## Read the data set with monthly volume, SC and elevation variables
mead.monthly<-
  read.table("Lake_Mead_volume_SC_elevation_time_series_monthly.csv",
             header=T,sep=",")
mead.monthly
attach(mead.monthly)
vol=vol
SC=SC
SC
detach(mead.monthly)

#### Combine the variables of interest into a data frame
### Notice that meprobamate has missing values in Jan, Feb 2006, Jan 2007. 

  chemicals1=data.frame(meprobamate, vol,SC)

  chemicals1
   str(chemicals1)

### Make the dataset at zoo class data

  chemicals.zoo=zoo(chemicals1, 
                  order.by=date, 
                  frequency=12)

  chemicals.zoo
  str(chemicals.zoo)

  class(chemicals.zoo)
  frequency(chemicals.zoo)

## Check whether some observations missing

  is.regular(chemicals.zoo,strict=TRUE)


###This plot shows the missing values 

#png("wastewaterdata.png", units="in", width=8.5, height=8, res=1200) # commented out
 par(
    mfrow=c(1,1),
    font.axis=2, 
    mar=c(5,5,5,5),
  font.main=2,
  font.lab=2
  )
plot.zoo(chemicals.zoo, lwd=1.5,cex=0.5, 
           main="Variables studied in this\n case study", 
          xlab="Time")
#  dev.off()  Run this to make the window disappear. 



# this next commented plot, due to the scale will not look too good. 
# dygraph(chemicals.zoo[,c("SC","meprobamate")], main = "SC and Meprobromate 2003-2007")   # added title
 
## This next dygraph shows the not so clear inverse relation between 
## chemical concentration and water volum. 
 dygraph(chemicals.zoo[,c("vol","meprobamate")], main = "Vol and Meprobromate 2003-2007") # added title and changed "elevation" to "vol"

  zoo::index(chemicals.zoo) # added zoo::
  names(chemicals.zoo)

##### Start analysis. Compare long term 
#### trends of volume and concentration
### Use stl()
  head(chemicals.zoo)
  chemicals.zoo$meprobamate

#### meprobamate decomposition
   a=stl(ts(chemicals.zoo$meprobamate,start=c(2003,3),freq=12),
      na.action = na.approx,s.window=29)
   a
#png("meprobamate.png", units="in", width=8.5, height=8, res=1200) # commented out
 par(
  mfrow=c(1,1),
  font.axis=2, 
  mar=c(5,5,5,5),
  font.main=2,
  font.lab=2
)
plot(a,main="Lowess decomposition of meprobamate\n using stl()",
     lwd=1.5,cex=0.5)
  dev.off()
  concentration.s=a$time.series[,"seasonal"]
 concentration.t=a$time.series[,"trend"]
a.r=a$time.series[,"remainder"]

#png("volume.png", units="in", width=8.5, height=8, res=1200) # commented out
par(
  mfrow=c(1,1),
  font.axis=2, 
  mar=c(5,5,5,5),
  font.main=2,
  font.lab=2
)

### Volume decomposition 
  b=stl(ts(chemicals.zoo$vol,start=c(2003,3),freq=12),
        na.action = na.approx,s.window=29)
 plot(b,main="Lowess decomposition of volume\n using stl()",
     lwd=1.5,cex=0.5)
 dev.off()

  volume.s=b$time.series[,"seasonal"]
  volume.t=b$time.series[,"trend"]; 
  b.r=b$time.series[,"remainder"]
  


  ## We may find a high negative contemporary correlation between concentration 
  ## and volume, but this is not the righ correlation to use when 
  # working with time series. So the next number should not 
  ## be considered. The data are richer than this simple number shows.
  ## We will learn cross-correlation, used by the Wildman, in a future chapter. 
 cor(volume.t, concentration.t) #fixed cor(b.t,a.t) to cor(volume.t, concentration.t)

### SC decomposition
  c=stl(ts(chemicals.zoo$SC,start=c(2003,3),freq=12),na.action = na.approx,s.window=29)

#png("SC.png", units="in", width=8.5, height=8, res=1200) # commented out
 par(
  mfrow=c(1,1),
  font.axis=2, 
  mar=c(5,5,5,5),
  font.main=2,
  font.lab=2
  )
plot(c,main="Lowess decomposition of SC\n using stl()",
     lwd=1.5,cex=0.5)
# dev.off() type this if you want the plots to not show on plot window
  SC.s=c$time.series[,"seasonal"]; length(SC.s)
  c.t=c$time.series[,"trend"]; length(c.t)
  c.r=c$time.series[,"remainder"]; length(c.r)
  

  ## The following are not overlay plots, they show the data as is. 

 plot(SC.s, xlab="Date", ylab="Values");lines(concentration.s,col="blue") # added legend, labels, and title 
title(main="SC and meprobromate seasonality, \n meprobromate peaks very close to SC");
legend("topleft", legend=c("SC.s", "Concentration.s"), col=c("black", "blue"), lty=1, cex=0.45)

plot(concentration.s, col="blue", xlab="Date", ylab="Values");lines(volume.s) # added legend, labels, and title 
title(main="Meprobromate and Volume Seasonality meprobromate\n leads Volume");
legend("topleft", legend=c("Volume", "Concentration"), col=c("black", "blue"), lty=1, cex=0.45)


## These two plots are like those in Figure 8 in the article by Wildman. 
plot(concentration.t, col="blue",ylim=c(5,21), xlab="Date", ylab="Values");lines(volume.t) # added legend, labels, and title 
title(main="Volume and Meprobromate Trend, Inverse Trends,\n As meprobromate increases, volume decreases");
legend("topleft", legend=c("Volume.t", "Concentration.t"), col=c("black", "blue"), lty=1, cex=0.45)

plot(x=concentration.t, y=volume.t, xlab="Meprobromate.t", ylab="Volume.t", cex=0.6) # added labels, and title, and cex=0.6
title(main="Meprobromate vs Volume Trend, 2005-2006\n Meprobromate increased despite increase Volume")


## These next two plots are not very useful due to the scales. Overlay plots could be more useful
plot(volume.t, ylim=c(15,21), xlab="Date", ylab="Volume.t") # added legend, labels, ylim= 15-21 to show fluctuations, and title 
title(main="Vol decreases 2003-late 2004 then increases till \nmid 2005 and then falls through 2008");
legend("topright", legend=c("Volume"), col=c("black"), lty=1, cex=0.45)

plot(c.t, ylim=c(0,1080), xlab="Date", ylab="Values");lines(volume.t,col="red");lines(concentration.t,col="blue")  # changed c.t/d.t to volume.t and concentration.t added legend, labels, and title
title(main="Trends of Meprobromate, Volume, and SC");legend("right", legend=c("SC.t", "Volume.t", "Concentration.t"), col=c("black","red","blue"), lty=1, cex=0.45)

       ########### Overlay plots ######## 

## Overlay plots present two vertical scales. So they may not appear to the naked 
## eye as if they represent the actual data. Figure 8 in Wildman's paper on the 
## left is an overlay plot. We reproduce it here. 

##overlay plot of SC and chemical
# overlay plot of seasonal trends of chemicals and Mead SC
par(mar=c(5,4,2,8)+0.1,cex.axis=0.85)
plot(concentration.s, # changed from a.s to concentration.s
     ylim=c(-8,8),
     xlab="",ylab=" ",col="red",lty=2)	
par(new=T, mar=c(5,4,2,8)+0.1)  		

plot(SC.s, # changed from c.s to SC.s
    ylim=c(-40,40),
     xlab="",ylab="",axes=F, col="blue")
axis(4)
mtext("meprobamate (ng/L)",cex=0.85,side=2,line=2.4)
mtext(expression(paste("Lake Mead SC (",mu,"s/cm^3)")), #???should it be cm^3 or just cm
      cex=0.85,side=4,line=2.4)
legend("bottom",cex=0.75,inset=0.02,
       c("meprobamate","specific conductance"),
       lty=c(2,1),col=c("red","blue"))


### we get similar plot to that of the authors. 

